import bb.cascades 1.0
import custom.lib 1.0
import bb.system 1.0

ListView {
    id: idListView
    horizontalAlignment: HorizontalAlignment.Fill
    property variant identitiesObj: identities
    signal listIdentitySelected(string id)
    signal showList()
    signal hideList()
    signal listItemSelected(string selectedId, bool isSelected)
    signal listIdentityEdit(string selectedId)
    signal defaultChanged(string id)
    signal nameChanged(string id, string name)
    dataModel: ArrayDataModel {
        id: idArrayDataModel
        onItemAdded: {
            printList()
            if (idArrayDataModel.childCount(0) == 1) {
                showList()
            }
        }
        onItemRemoved: {
            if (idArrayDataModel.childCount(0) == 0) {
                hideList()
            }
        }
    }
    
    onCreationCompleted: {
        //connect our listview to identites signal which gets
        //called when an identity is added
        identities.identityAdded.connect(idListView.onCPPIdentityAdded)
        identities.identityRemoved.connect(idListView.onCPPIdentityRemoved)
        identities.identityOrderChanged.connect(idListView.onCPPIdentityOrderChanged)
        identities.identityOrderSwapped.connect(idListView.onCPPIdentityOrderSwapped)
        identities.defaultIdentityChanged.connect(idListView.defaultChanged)
        identities.nameChanged.connect(idListView.onCPPIdentityNameChanged)
    }
    //slot for identity being added
    function onCPPIdentityAdded(name, id, order, imgSource) {
        
        //insert item into the group data model
        idArrayDataModel.insert(order,{
                "name": name,
                "id": id,
                "order": order,
                "imgSource": imgSource
        });
        
    }
    function onCPPIdentityRemoved(id) {

        console.log("removing identity with id: " + id);

        for(var i=0; i < dataModel.childCount(0); i++) {
            
            var identity = dataModel.value(i);
            //check to see whether the identity is the deleted one
            if (identity.id === (id)) {
                //remove the identity
                dataModel.removeAt(i);
                return;
            } 
        }
    }
    function onCPPIdentityOrderChanged(id, newOrder) {
        
        for(var i=0; i < dataModel.childCount(0); i++) {
            
            var identity = dataModel.value(i);
            //check to see whether the identity is the deleted one
            if (identity.id === (id)) {
                //remove the identity
                dataModel.removeAt(i);
                dataModel.insert(newOrder,identity);
                return;
            } 
        }
    }
    function onCPPIdentityOrderSwapped(firstOrder, secondOrder) {
        
        dataModel.swap( firstOrder, secondOrder)
        
    }
    function onCPPIdentityNameChanged(id, name)
    {
        nameChanged(id, name)
    }
    // Override the itemType() function to return the proper type
    function itemType(data, indexPath) {
    	return "item";
    }
    listItemComponents: [
        IdentityListItem {
            id: identityItemComponent
            type: "item"
        } // end of second ListItemComponent
    ] //end list item components

    multiSelectHandler {
        status: qsTr("None selected") + Retranslate.onLanguageChanged
        // The actions that can be performed in a multi-select sessions are set up in the actions list.
        actions: [
            DeleteActionItem {
                onTriggered: {
                    var i;
                    var selectedIdentities = selectionList();
                    for (i = 0; i < selectedIdentities.length; i++) {
                        var selectedItem = dataModel.data(selectedIdentities[i]);
                        listItemSelected(selectedItem.id, false)
                    }
                    
                    identities.deleteIdentities(selectedIdentities, dataModel)                     
                }
            }
        ]
    }
    attachedObjects: [
        SystemDialog {
            property variant identityId: null
            property variant identityName: null
            id: deleteIdentityDialog
            title: qsTr("Remove Identity") + Retranslate.onLanguageChanged
            body: qsTr("Are you sure you want to remove the identity %0?").arg(identityName) + Retranslate.onLanguageChanged
            onFinished: {
                if (deleteIdentityDialog.result == SystemUiResult.ConfirmButtonSelection) {
                    identities.deleteIdentity(identityId)
                }
            }
        }
    ]
    onSelectionChanged: {
        var selectedItem = dataModel.data(indexPath);
        listItemSelected(selectedItem.id, selected)
        updateMultiStatus();
    }
    onTriggered: {
        var selectedItem = dataModel.data(indexPath);
        if (multiSelectHandler.active == true) {
            selectedItem.selected = true
            select(indexPath, true)
            console.log("selecting item" + selectedItem.id);
        } else {
            console.log(selectedItem.id);
            listIdentitySelected(selectedItem.id);
        }

    }
    function updateMultiStatus() {

        // The status text of the multi-select handler is updated to show how
        // many items are currently selected.
        if (selectionList().length > 0) {
            multiSelectHandler.status = identities.translatePluralItemsSelected(selectionList().length);
        } else {
            multiSelectHandler.status = qsTr("None selected");
        }
    }
    function deleteIdentity(id, name) {
        console.log("id: " + id);
        deleteIdentityDialog.identityId = id
        deleteIdentityDialog.identityName = name
        deleteIdentityDialog.show();
    }
    function editIdentity(id) {
        listIdentityEdit(id)
    }
    function setDefaultIdentity(id) {
        identities.setDefaultIdentity(id)
    }

    function getDefaultIdentity() {
        return identities.getDefaultIdentityId();
    }
    function increaseOrder(id) {
        identities.increaseOrder(id)
        
    }
    function decreaseOrder(id) {
        identities.decreaseOrder(id)
    }
    function getNameAlignment()
    {
        return style.identityListNameAlignment;
    }
    function getDefaultAlignment()
    {
        return style.identityListDefaultAlignment;
    }
    function printList()
    {
        var indexPath = 0;
        var i = 0;
        
        while (i < dataModel.childCount(0)) {
            indexPath = [ i ];
            var identity = dataModel.data(indexPath);
            console.log("id equals: " + identity.id  );
            
            
            i++;
        }
    }
}//end list view
