import bb.cascades 1.0
import custom.lib 1.0
import "../../../Styles/Colors.js" as Colors
ListItemComponent {
    id: identityItem

    //outline container
    IdentitiesListItemOutsideContainer {
        property variant backgroundColor: Color.create(Colors.backgroundLight)
        property variant highlightBackgroundColor: Color.create(Colors.selectedBlue)
        property variant selectedBackgroundColor: Color.create(Colors.highlightBlue)
        id: identityItemContainer
        preferredWidth: Infinity
        verticalAlignment: VerticalAlignment.Fill
        horizontalAlignment: HorizontalAlignment.Fill
        background: backgroundColor

        layout: DockLayout {

        }
        leftPadding: 10.0
        bottomPadding: 10.0
        rightPadding: 10.0
        topPadding: 10.0
        //actual container for list items
        IdentitiesListItemInsideContainer {

            background: Color.White
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill

            layout: DockLayout {

            }

            Label {
                id: nameLabel
                text: ListItemData.name
                textStyle.fontSize: FontSize.PointValue
                textStyle.fontSizeValue: 16.0
                textStyle.color: Color.Black
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: identityItemContainer.ListItem.view.getNameAlignment()
                leftMargin: 20.0

            }
            WebImageView {
                id: logoView
                verticalAlignment: VerticalAlignment.Center
                horizontalAlignment: identityItemContainer.ListItem.view.getNameAlignment()
                leftMargin: 20.0
                url: ListItemData.imgSource
                scalingMethod: ScalingMethod.AspectFit
                attachedObjects: [
                    LayoutUpdateHandler {
                        onLayoutFrameChanged: {
                            logoView.preferredHeight = layoutFrame.height
                        }
                    }
                ]
                onImageChanged: {
                    nameLabel.setVisible(! isValid)
                }
            }
            Label {
                id: defaultLabel
                text: qsTr("Default") + Retranslate.onLanguageChanged
                horizontalAlignment: identityItemContainer.ListItem.view.getDefaultAlignment()
                textStyle.fontSize: FontSize.PointValue
                textStyle.fontSizeValue: 8.0
                textStyle.color: highlightBackgroundColor
                verticalAlignment: VerticalAlignment.Bottom

                visible: {
                    identityItemContainer.ListItem.view.getDefaultIdentity() == ListItemData.id;
                }
            }

            contextActions: [
                ActionSet {
                    actions: [
                        ActionItem {
                            id: actionUp
                            title: qsTr("Move Up")
                            imageSource: "asset:///images/BBActions/ic_to_top.png"
                            onTriggered: {
                                identityItemContainer.ListItem.view.decreaseOrder(ListItemData.id);
                            }

                        },
                        ActionItem {
                            id: actionDown
                            title: qsTr("Move Down")
                            imageSource: "asset:///images/BBActions/ic_to_bottom.png"
                            onTriggered: {
                                identityItemContainer.ListItem.view.increaseOrder(ListItemData.id);
                            }

                        },
                        ActionItem {
                            id: actionEdit
                            title: qsTr("Edit")
                            imageSource: "asset:///images/BBActions/ic_edit.png"
                            onTriggered: {
                                identityItemContainer.ListItem.view.editIdentity(ListItemData.id);
                            }
                        },
                        ActionItem {
                            id: actionDefault
                            title: qsTr("Make Default")
                            imageSource: "asset:///images/BBActions/ic_select.png"
                            onTriggered: {

                                identityItemContainer.ListItem.view.setDefaultIdentity(ListItemData.id);
                            }

                        },
                        MultiSelectActionItem {
                            multiSelectHandler: identityItemContainer.ListItem.view.multiSelectHandler

                            onTriggered: {
                                multiSelectHandler.active;
                            }
                        },
                        DeleteActionItem {
                            id: actionDelete
                            onTriggered: {
                                background = backgroundColor
                                identityItemContainer.ListItem.view.deleteIdentity(ListItemData.id, ListItemData.name);
                            }
                        }
                    ]
                } // end of ActionSet
            ]
        }
        onCreationCompleted: {
            identityItemContainer.ListItem.view.listItemSelected.connect(identityItemContainer.setSelected)
            identityItemContainer.ListItem.view.defaultChanged.connect(identityItemContainer.setDefault)
            identityItemContainer.ListItem.view.nameChanged.connect(identityItemContainer.nameChanged)

        }
        function setSelected(selectedId, isSelected) {
            if (selectedId === ListItemData.id) {

                if (isSelected == true) {
                    background = selectedBackgroundColor
                } else {
                    background = backgroundColor
                }
            }
        }
        function setDefault(id) {
            if (id === ListItemData.id) {
                defaultLabel.setVisible(true)
            } else {
                defaultLabel.setVisible(false)
            }
        }
        function nameChanged(id, name) {
            console.log("name changed")
            if (id === ListItemData.id) {
                nameLabel.text = name
            }
        }
    }

}