import bb.cascades 1.0
import custom.lib 1.0
import "../Components/AddIdentity"
import "../Components/Shared"
import "../../Styles/Colors.js" as Colors
/**
 * The Add Identity page provides a way for the user to create an association between
 * theirself and a bank account
 */
ContentContainer {
    property alias save: saveButton
    property alias cancel: cancelButton
    name: qsTr("Add Identity") + Retranslate.onLanguageChanged
    contents: addIdentityContainer
    rightObject: saveButton
    leftObject: cancelButton
    background: Color.create(Colors.backgroundDark)
    
    onCreationCompleted: {
        addIdentityCPPPage.clearFields.connect(clearFields)
        QA.invokeQuickActivationStartup.connect(setQuickActivationFields);
        QA.invokeQuickActivationInvalid.connect(clearFields);
    }
    attachedObjects: [
        Button {
            id: cancelButton
            objectName: "cancelButton"
            text: qsTr("Cancel") + Retranslate.onLanguageChanged
            preferredWidth: 200
        },
        Button {
            id: saveButton
            objectName: "saveButton"
            text: qsTr("Save") + Retranslate.onLanguageChanged
            preferredWidth: 200
            onClicked: {
                saveIdentity();
            }
        },
        //container with the header and form data
        Container {
            id: addIdentityContainer
            layout: StackLayout {
            }


            horizontalAlignment: HorizontalAlignment.Fill


            //container for a form containing the Identity Provider Information
            Container {
                id: identityProviderInfoContainer
                //background: Color.create(Colors.header)
                background: Color.create(Colors.backgroundDark)
                
                bottomPadding: 30
                Container {
                    
                    background: Color.create(Colors.backgroundDark)
                    Container {
                        leftPadding: 20.0
                        rightPadding: 20.0
                        topPadding: 20.0
                        Container {
                            layout: StackLayout {


                            }
                            Label {
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 10.0
                                text: qsTr("Identity Provider") + Retranslate.onLanguageChanged
                                horizontalAlignment: style.addIdentityLabelAlignment
                            }

                            // Quick activation summary message (optional, off by default)
                            Label {
                                id: quickActivationTitle   
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0                                
                                text: qsTr("Activation Summary") + Retranslate.onLanguageChanged                                
                                horizontalAlignment: style.addIdentityLabelAlignment
                                visible: false;                        
                            }                            
                            TextArea {
                                id:  quickActivationMsg
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                text: qsTr("You are about to activate a new soft token identity. This identity will automatically register with your identity provider.  To begin activation, click the Save button.") + Retranslate.onLanguageChanged
                                visible: false;
                                editable: false;
                            }

                            Label {
                                id:  addressFieldLabel
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                text: qsTr("Address:") + Retranslate.onLanguageChanged
                                horizontalAlignment: style.addIdentityLabelAlignment
                            }
                            TextField {
                                id: addressField
                                hintText: qsTr("example.com") + Retranslate.onLanguageChanged
                                objectName: "addressField"
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                validator: Validator {
                                    objectName: "addressValidator"
                                    mode: ValidationMode.FocusLost


                                    onValidate: {
                                        //combine fields
                                        var address = addressField.text;


                                        //validate whether they are correct
                                        addIdentityCPPPage.setIdentityAddress(address)
                                    }
                                }
                                inputMode: TextFieldInputMode.Url
                            }
                        }
                    }
                    Container {
                        topPadding: 10
                        leftPadding: 20.0
                        rightPadding: 20.0
                        bottomPadding: 60.0
                        Container {
                            layout: StackLayout {


                            }
                            Label {
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                text: qsTr("Name:") + Retranslate.onLanguageChanged
                                horizontalAlignment: style.addIdentityLabelAlignment
                            }
                            TextField {
                                id: identityName
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                objectName: "identityName"
                                hintText: qsTr("Identity Provider Name") + Retranslate.onLanguageChanged
                                onFocusedChanged: {
                                    if (focused == false) {
                                        serialNumber.setFocus();
                                    }
                                }
                            }
                            
                            
                            // For quick activation, display the Address fields
                            // and serial numbers in this part of the display
                            Label {
                                id: quickActivationAddressLabel
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                text: qsTr("Address:") + Retranslate.onLanguageChanged
                                horizontalAlignment: style.addIdentityLabelAlignment
                                visible: false
                            }
                            TextField {
                                id: quickActivationAddressValue
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                visible: false
                            }
                            Label {
                                id: quickActivationSerialNumberLabel
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                text: qsTr("Serial Number:") + Retranslate.onLanguageChanged
                                horizontalAlignment: style.addIdentityLabelAlignment
                                visible: false
                            }
                            TextField {
                                id: quickActivationSerialNumberValue
                                textStyle.fontSize: FontSize.PointValue
                                textStyle.fontSizeValue: 8.0
                                visible: false
                            }
                            
                        }
                    }
                }
            }
            //registration info container
            Container {
                id: registrationNumberInfoContainer
                topMargin: 20
                leftPadding: 20.0
                rightPadding: 20.0
                layout: StackLayout {


                }
                //Soft Token Label
                Label {
                    textStyle.fontSize: FontSize.PointValue
                    textStyle.fontSizeValue: 10.0
                    text: qsTr("Soft Token") + Retranslate.onLanguageChanged
                    horizontalAlignment: style.addIdentityLabelAlignment
                }
                //serial number container
                Container {
                    Container {
                        id: serialNumberContainer


                        layout: StackLayout {
                        }
                        MediumWhiteLabel {
                            textStyle.fontSize: FontSize.PointValue
                            textStyle.fontSizeValue: 8.0
                            text: qsTr("Serial Number:") + Retranslate.onLanguageChanged
                            horizontalAlignment: style.addIdentityLabelAlignment
                        }
                        SerialNumber {
                            id: serialNumber
                            nextFocus: activationCode.firstFocus
                        }
                    }
                }


                Container {
                    topPadding: 10
                    Container {
                        //activation code container
                        id: activationCodeContainer
                        layout: StackLayout {
                        }
                        bottomPadding: 20.0
                        Label {
                            textStyle.fontSize: FontSize.PointValue
                            textStyle.fontSizeValue: 8.0
                            text: qsTr("Activation Code:") + Retranslate.onLanguageChanged
                            horizontalAlignment: style.addIdentityLabelAlignment
                        }
                        ActivationCode {
                            id: activationCode
                        }


                    }


                }
            }


        } //end addIdentity container
    ] //end attachedObjects


    function saveIdentity() {
        var address = addressField.text;
        var sn = serialNumber.getCode();
        var ac = activationCode.getCode();
        var name = identityName.text;
        
        var valid = addIdentityCPPPage.submit(address, name, sn, ac);


    }
    function getName() {
        return identityName.text
    }
    function clearFields() {
        console.log("clearing fields");
        addressField.text = "";
        identityName.text = "";
        activationCode.clear();
        serialNumber.clear();
        
        resetQuickActivationFields();
    }
    
    function setQuickActivationFields() {
        // This is invoked when we need to pre-set our field values
        // based on input data that were received from the 
        // quick activation URL
        
        // Set the serial number
        serialNumber.setCode(QA.serialNumber0, QA.serialNumber1);
        
        // Set the address field, but don't make it editable
        addressField.text = QA.address;
        addressField.visible = false;
        // show the summary message        
        quickActivationTitle.visible = true;
        quickActivationMsg.visible = true;
        // show the quick activation details (address and serial number)
        quickActivationAddressLabel.visible = true;                        
        quickActivationAddressValue.visible = true;
        quickActivationAddressValue.text = QA.address;
        quickActivationAddressValue.opacity = 0.85
        quickActivationAddressValue.enabled = false;
        quickActivationSerialNumberLabel.visible = true;                        
        quickActivationSerialNumberValue.visible = true;
        quickActivationSerialNumberValue.text = QA.serialNumber0 + "-" + QA.serialNumber1;;
        quickActivationSerialNumberValue.opacity = 0.85
        quickActivationSerialNumberValue.enabled = false;
        // Hide all the other fields
        activationCodeContainer.visible = false;
        registrationNumberInfoContainer.visible = false;
        addressFieldLabel.visible = false;
    }

    function resetQuickActivationFields() {
        // When exiting from this pane, we should reset 
        // the form to not include the quick activation related data
        
        // Hide all quick activation related fields
        quickActivationTitle.visible = false;
        quickActivationMsg.visible = false;
        quickActivationAddressLabel.visible = false;                        
        quickActivationAddressValue.visible = false;
        quickActivationSerialNumberLabel.visible = false;                        
        quickActivationSerialNumberValue.visible = false;
        
        // Show the original fields (which were previously hidden when quick
        // activation was enabled)
        activationCodeContainer.visible = true;
        registrationNumberInfoContainer.visible = true;
        addressFieldLabel.visible = true;
        addressField.visible = true;        
    }
}//end page
