// Default empty project template
import bb.cascades 1.0
import custom.lib 1.0
import bb.system 1.0
import "Pages"
import "Components/Shared"

/**
 * This is the main En, Es, Fr language layout for the Z10 device
 */
NavigationPane {
    id: navigationPane    
    property bool sheetOpened:false
    property variant openSheet: undefined //Keeps track of the type of sheet that is displayed on top when the app comes to foreground

    onPushTransitionEnded: {
        setPageTransition()
    }
    onPopTransitionEnded: {
        setPageTransition()
        
        if(page == infoPage) {
        	infoItem.setEnabled(true)
        }
        
        if(page == globalSettingsPage) {
            settingsItem.setEnabled(true);
        }
    }
    function setPageTransition() {
        peekEnabled = true;

        if (navigationPane.top == identitiesPage || navigationPane.top == globalSettingsPage || navigationPane.top == useIdentityPage) {
            setMainMenu();
        } else if (navigationPane.top == pinEntryPage || navigationPane.top == pinCreationPage) {
            /*
             * If a sheet is opened, it will open the the full keyboard before opening the numeric keyboard
             * so we prevent the keyboard from opening immediately when a sheet is opened the app comes to foreground
             */
            if(!navigationPane.sheetOpened) {                
                identities.showKeyboard();
            }
            setPinMenu();            
            peekEnabled = false;
        } else if (navigationPane.top == pinUnlockPage) {
            setUnlockMenu();
            identities.showKeyboard();
            peekEnabled = false;
        } else if (navigationPane.top == infoPage) {
            setInfoMenu();
        }
    }

    Menu.definition: mainMenu
    function setInfoMenu() {
        mainMenu.removeAction(advancedAction)
    }
    function setMainMenu() {
        mainMenu.removeAction(advancedAction)
        mainMenu.setSettingsAction(settingsItem)
    }
    function setUnlockMenu() {
        mainMenu.addAction(advancedAction)
        mainMenu.setSettingsAction(null)
    }
    function setPinMenu() {
        mainMenu.removeAction(advancedAction)
        mainMenu.setSettingsAction(null)
    }
    onCreationCompleted: {
        identities.applicationPinLocked.connect(navigationPane.applicationPinLocked);
        identities.invalidIdentityLoaded.connect(navigationPane.invalidIdentityLoaded)
        identities.useIdentity.connect(navigationPane.openUseIdentity)
        Application.awake.connect(onFullscreen);
        Application.asleep.connect(onSleep);
        appui.incorrectEncryptionDevice.connect(incorrectEncryptionDevice)
        QA.invokeQuickActivationStartup.connect(doQuickActivation);
        QA.invokeQuickActivationInvalid.connect(doQuickActivationInvalid);
    }
    
    function doQuickActivation() {
        // For quick activation, restart at the main application page.
        // If required, pop off any pages that are currently on top
        var numpages = navigationPane.count()-1;
        var i=0;
        for (i=0; i<numpages; i++) {
            navigationPane.pop();
        }   
        // Now we can open the add identity page     
        openAddIdentity("");        
    }

    function doQuickActivationInvalid() {
        // For quick activation, restart at the main application page.
        // If required, pop off any pages that are currently on top
        var numpages = navigationPane.count()-1;        
        var i=0;
        for (i=0; i<numpages; i++) {
            navigationPane.pop();
        }   

        // sheets have to be explicitly closed as well, so that we
        // end up at the main identities page
        if (openSheet != undefined) {
            console.log("Received invalid Quick Activation link, close open sheet " + openSheet)
            navigationPane.openSheet.close();
            navigationPane.openSheet = undefined;
            sheetOpened = false;
        }        

        toast.body = qsTr("The link provided is invalid.  Please return to your identity provider and attempt the action again.") + Retranslate.onLanguageChanged
        toast.show();        
    }
    
    function applicationPinLocked(isLocked) {

        if (isLocked == true) {
            pinEntryPage.push()
        }
    }
    function onFullscreen() {        
        if (identities.isPinEnabled() == true) {
            if (!pinEntryPage.opened) {
                /*
                 * As soon as the app comes to foreground, check if a sheet is open so we can close it and reopen it 
                 * after entering the correct pin. The sheet will prevent the 'Pin Entry Page' from being pushed to top
                 * of the stack.
                 */
                if(registrationCodeSheet.opened) {
                    navigationPane.sheetOpened = true;
                    registrationCodeContainer.done.clicked();                    
                }  
                else if(addIdentitySheet.opened){                 
                    navigationPane.sheetOpened = true;
                    navigationPane.openSheet = addIdentitySheet;
                    addIdentitySheet.close();
                } 
                else if (identitySettingsSheet.opened) {                    
                    navigationPane.sheetOpened = true;
                    navigationPane.openSheet = identitySettingsSheet;
                    identitySettingsSheet.close();
                }
                pinEntryPage.push();                
                console.log("top is not pin entry page")
            } else {
                identities.showKeyboard()
            }
        }
    }
    function onSleep() {
        // Why was this needed?
        //if (top == pinEntryPage) {
        //     identities.hideKeyboard()
        //}
    }
    function openBrowser(uri) {
        invoke.query.uri = uri
        invoke.trigger("bb.action.OPEN")
    }
    function editIdentity(selectedId) {
        if(identities.isIdentityActivated(selectedId)=== true)
        {
            console.log("activated")
            identitySettings.setIdentity(selectedId);
            identitySettingsSheet.open()
        }
        else
        {
            console.log("not activated")
            openAddIdentity(selectedId)
        }
    }
    function invalidIdentityLoaded(identityName) {
        toast.body = qsTr("Failed to load identity %0 because the saved device ID does not match your current device ID.").arg(identityName) + Retranslate.onLanguageChanged
    }
    function incorrectEncryptionDevice() {
        toast.body = qsTr("Unable to load data from the device because the saved device ID does not match your current device ID.") + Retranslate.onLanguageChanged
    }
    function openUseIdentity(identityId) {
        if(identities.isIdentityActivated(identityId) === true)
        {
            //if activated we can view otp
            navigationPane.push(useIdentityPage);
            useIdentity.startTimer(identityId);
        }
        else
        {
           openAddIdentity(identityId)
        }
    }
    function openAddIdentity(identityId)
    {
        addIdentityContainer.setIdentity(identityId);
        addIdentitySheet.open();
    }
    //main identity management page is the root of our applications navigation pane
    Page {
        id: identitiesPage        
        actions: [
            ActionItem {
                id: addIdentity
                title: qsTr("Add New") + Retranslate.onLanguageChanged
                ActionBar.placement: ActionBarPlacement.OnBar
                imageSource: "asset:///images/BBActions/ic_add.png"

                onTriggered: {
                    console.log("opening add identity")
                    openAddIdentity("")
                }
            }
        ]
        IdentitiesPage {
            parentPage: identitiesPage
            onCreationCompleted: {
                idListView.listIdentitySelected.connect(listIdentitySelected)
                idListView.listIdentityEdit.connect(listIdentityEdit)
            }
            function listIdentityEdit(id) {
                editIdentity(id)
            }
            function listIdentitySelected(id) {
                navigationPane.openUseIdentity(id);
            }
        }
    }
    //all pages that are part of the application
    attachedObjects: [
    	DisplayInfo {
    	    id: displayInfo
    	},
    	Dialog {
            id: pinEntryPage

	        Container { 
	            id: entryDialogContainer
	            focusPolicy: FocusPolicy.KeyAndTouch

	            onFocusedChanged: {
	                if(!focused) {
	                    requestFocus()
	                } else {
	                    identities.showKeyboard();
	                }
	            }
	            keyListeners: [
	                NumberListener {
	                    onNumberPressed: {
	
	                        pinEntryContainer.pinTextChanging(number)
	                    }
	                    onBackPressed: {
	                        pinEntryContainer.backPressed();
	                    }
	                }
	            ]
	
	            PinEntry {
                    minHeight: displayInfo.pixelSize.height  // Make the dialog fill the screen
	                parentPage: null
	                id: pinEntryContainer
	                onPinValidated: {
	                    console.log("pin validated")
	                    pinEntryPage.pop()
	                    
                        // Only after we've validated the PIN should we
                        // start the quick activation process (if enabled)
                        QA.activateSignal();
                        
	                    
	                    /*
	                     * If a valid sheet was open before the app went to background,
	                     * reopen it and clear the openSheet property.
	                     */
	                    if(navigationPane.openSheet != undefined){
	                        navigationPane.openSheet.open()
	                        navigationPane.openSheet = undefined;
	                    } 

	                }
	                onPinLockedOut: {
	                    pinEntryPage.pop()
	                    pinUnlockContainer.setCode(identities.getUnlockChallenge());
	                    pinUnlockPage.push()
	                }
	            }
	        }
            function push() {
                pinEntryPage.open()
                pinEntryContainer.begin();
                identities.showKeyboard();
            }
            function pop() {
                identities.hideKeyboard();
                pinEntryPage.close()
            }
            onOpened: {
                entryDialogContainer.requestFocus()
            }
        },
        Page {
            id: pinCreationPage            
            actionBarVisibility: ChromeVisibility.Hidden
            keyListeners: [
                NumberListener {
                    onNumberPressed: {
                        pinCreationContainer.pinTextChanging(number)
                    }
                    onBackPressed: {
                        pinCreationContainer.backPressed();
                    }
                }
            ]
            PinCreation {
                id: pinCreationContainer
                parentPage: pinCreationPage
                onCanceled: {
                    pinCreationPage.pop();
                }
                onPinCreated: {
                    pinCreationPage.pop();
                }
            }
            function push() {
                navigationPane.push(pinCreationPage)
                pinCreationContainer.begin();
            }
            function pop() {
                pinCreationContainer.stop();
                navigationPane.pop(pinCreationPage)
            }
        },
        Page {
            id: globalSettingsPage            
            GlobalSettingsCPPPage {
                id: globalSettings
                onEditIdentitySelected: {
                    editIdentity(selectedId);
                }
                onDisablePin: {
                    //prompt the user to enter their pin
                    pinEntryContainer.pinValidated.connect(pinValidatedDisable)
                    pinEntryPage.push()
                }
                onNewPinRequired: {
                    if (identities.isPinEnabled() === true) {
                        //prompt the user to enter their pin
                        pinEntryContainer.pinValidated.connect(pinValidatedChange)
                        pinEntryPage.push()
                    } else {
                        pinCreationPage.push()
                    }
                }
                function pinValidatedChange() {
                    pinEntryContainer.pinValidated.disconnect(pinValidatedChange)
                    pinCreationPage.push()
                    pinCreationContainer.requestFocus();
                }
                function pinValidatedDisable() {
                    pinEntryContainer.pinValidated.disconnect(pinValidatedDisable)
                    //on to off
                    identities.disablePin();
                }
                onCreationCompleted: {
                    globalSettingsPage.setTitleBar(globalSettings.titleBar)
                }
            }
            paneProperties: NavigationPaneProperties {
                backButton: ActionItem {

                    onTriggered: {
                        navigationPane.pop();
                    }
                }
            }
        },
        Sheet {
            id: identitySettingsSheet
            peekEnabled: false
            content: Page {
                id: identitySettingsPage                
                IdentitySettingsCPPPage {
                    id: identitySettings

                    onSettingsClosed: {
                        console.log("closed");
                        identitySettingsSheet.close();
                        globalSettings.resetSelection();
                    }
                    onReactivate: {
                        reactivateIdentityDialog.identityId = identityId
                        reactivateIdentityDialog.show();
                    }
                    onCreationCompleted: {
                        identitySettingsPage.setTitleBar(identitySettings.titleBar)
                    }
                    onSaveError: {
                        toast.body = error;
                        toast.show();
                    }
                }
            }
            attachedObjects: SystemDialog {
                property variant identityId : ""
                id: reactivateIdentityDialog
                title: qsTr("Reactivate Identity") + Retranslate.onLanguageChanged
                body: qsTr("Reactivating this identity will prevent you from using it until you have completed the reactivation process. Are you sure?") + Retranslate.onLanguageChanged
                onFinished: {
                    if (reactivateIdentityDialog.result == SystemUiResult.ConfirmButtonSelection) {
                        identities.setIdentityActivated(identityId, false);
                        openAddIdentity(identityId);
                        identitySettingsSheet.close();
                    } else if (reactivateIdentityDialog.result == SystemUiResult.ConfirmButtonSelection) {
                        appResetDialog.cancel();
                    }
                }
            }
        },
        Sheet {
            id: addIdentitySheet
            peekEnabled: false
            content: Page {
                id: addIdentityPage                
                AddIdentityCPPPage {
                    id: addIdentityContainer
                    onSaved: {
                        addIdentitySheet.close();

                        navigationPane.push(useIdentityPage);
                        useIdentity.startTimer(identityId);

                    }
                    onCanceled: {
                        addIdentitySheet.close();
                    }
                    onDisplayRegistrationCode: {
                        registrationCodeContainer.setCode(registrationCode)
                        registrationCodeSheet.open();
                    }
                    onNewPinRequired: {
                        console.log("new pin required");

                        //listen for our pin created signal
                        pinCreationContainer.pinCreated.connect(addIdentityContainer.pinCreated);
                        addIdentitySheet.close()
                        pinCreationPage.push()
                        pinCreationContainer.showKeyboard()
                    }
                    function pinCreated(pin) {
                        //disconnect our signals
                        pinCreationContainer.pinCreated.disconnect(addIdentityContainer.pinCreated);

                        //save pin
                        addIdentityContainer.registerIdentity();
                    }
                    onCreationCompleted: {                        
                        addIdentityPage.setTitleBar(addIdentityContainer.titleBar)
                    }
                    onSaveError: {
                        toast.body = saveError;
                        toast.show();
                    }
                }                
            }
        },
        Page {
            id: useIdentityPage            
            OTPView {
                id: useIdentity
                onCreationCompleted: {
                    useIdentityPage.setTitleBar(useIdentity.titleBar)
                }
                onOtpTimeout: {
                    useIdentity.renewCode();
                }
            }
        },
        Page {
            id: infoPage            
            InformationControlCPPPage {
                id: infoControlContainer
                onPushed: {
                    console.log("pushed");
                    navigationPane.push(page)
                }
                onCreationCompleted: {
                    infoPage.setTitleBar(infoControlContainer.titleBar)
                }
            }
            paneProperties: NavigationPaneProperties {
                backButton: ActionItem {

                    onTriggered: {
                        navigationPane.pop();
                    }
                }
            }

        },
        Sheet {
            id: registrationCodeSheet
            peekEnabled: false
            content: Page {
                id: registrationCodePage
                RegistrationCode {
                    parentPage: registrationCodePage
                    id: registrationCodeContainer

                    done.onClicked: {                        
                        registrationCodeSheet.close();
                        addIdentityContainer.saveIdentity();
                    }
                }
            }
        },
        Page {
            id: pinUnlockPage
            actionBarVisibility: ChromeVisibility.Hidden
            PinUnlock {
                parentPage: pinUnlockPage
                id: pinUnlockContainer
                onUnlocked: {
                    navigationPane.pop(pinUnlockPage);
                    pinCreationPage.push();
                }
            }
            onCreationCompleted: {
                pinUnlockPage.titleBar.scrollBehavior = TitleBarScrollBehavior.Sticky
            }
            function push() {
                navigationPane.push(pinUnlockPage)
            }

        },
        SystemDialog {
            id: appResetDialog
            title: qsTr("Reinitialize") + Retranslate.onLanguageChanged
            body: qsTr("Warning. Reinitializing the application removes all information. Do you want to continue?") + Retranslate.onLanguageChanged
            onFinished: {
                if (appResetDialog.result == SystemUiResult.ConfirmButtonSelection) {
                    // make sure we aren't displaying any identities or pages
                    addIdentityContainer.reset();                    
                    if(navigationPane.openSheet != undefined){
                        console.log("[main.qml::SystemDiagl] closing open sheet")
                        navigationPane.openSheet.close();
                        navigationPane.openSheet = undefined;
                        sheetOpened = false;
                    } 
                    var numpages = navigationPane.count()-1;
                    var i=0;
                    for (i=0; i<numpages; i++) {                        
                        navigationPane.pop();
                    }   

                    identities.resetAppData();
                    appResetDialog.cancel();
                    
                    toast.body = qsTr("You have successfully reinitialized IdentityGuard Mobile.") + Retranslate.onLanguageChanged
                    toast.show();        
                    
                } else if (appResetDialog.result == SystemUiResult.CancelButtonSelection) {
                    appResetDialog.cancel();
                }
            }
        },
        // Add the application menu using a MenuDefinition
        MenuDefinition {
            id: mainMenu
            
            // Specify the actions that should be included in the menu
            helpAction: HelpActionItem {
                id: infoItem
                title: qsTr("Info") + Retranslate.onLanguageChanged
                onTriggered: {
                    if (enabled) {
                        navigationPane.push(infoPage)
                        setEnabled(false)
                    }
                }
            }
            actions: [
                ActionItem {
                    title: qsTr("Invite") + Retranslate.onLocaleOrLanguageChanged
                    imageSource: "asset:///images/ic_bbm.png"
                    onTriggered: {
                        _appShare.shareApp();
                    }
                }
            ]

        }, // end of MenuDefinition
        SettingsActionItem {
            id: settingsItem
            title: qsTr("Settings") + Retranslate.onLanguageChanged
            imageSource: "asset:///images/BBActions/ic_settings.png"
            onTriggered: {
                if (enabled) {
                    navigationPane.push(globalSettingsPage)
                    setEnabled(false)
                }
            }
        },
        ActionItem {
            id: advancedAction
            title: qsTr("Advanced") + Retranslate.onLanguageChanged
            ActionBar.placement: ActionBarPlacement.OnBar
            imageSource: "asset:///images/BBActions/ic_advanced.png"
            onTriggered: {
                appResetDialog.show();
            }
        },
        Invocation {
            id: invoke
            property variant uriSet: false
            query {
                invokeTargetId: "sys.browser"
                mimeType: "application/xhtml+xml"
                invokeActionId: "bb.action.OPEN"
                onUriChanged: {
                    uriSet = true
                    invoke.query.updateQuery();
                }
            }
            onArmed: {
                if (uriSet == true) {
                    invoke.trigger("bb.action.OPEN")
                }
            }
        },
        SystemToast {
            id: toast
        }

    ]
}//end main NavigationPane
